@@ -2,7 +2,9 @@ import numpy as np
 import pandas as pd
 import datetime as dt
 
-from . import dirs
+import requests
+from bs4 import BeautifulSoup as bs
+
 from . import fechas
 
 __all__ = ['BCRA',]
@@ -10,18 +12,23 @@ class BCRA():
 
     def __init__(self,
         cargar_tc = True,
+        cargar_tc_minorista = False,
         cargar_rem = True,):
         
+        self.__rem = None
         self.__tc_dia = None
         self.__tc_udh = None
         self.__tc_rem = None
-        self.__rem = None
+        self.__tc_min = None
         
         if cargar_tc:
             self.cargar_tc_ultimo_dia_habil(devolver=False)
             
         if cargar_rem:
             self.cargar_rem()
+            
+        if cargar_tc_minorista:
+            self.cargar_tc_minorista()
     
     @property
     def rem(self):
@@ -54,6 +61,14 @@ class BCRA():
     @tc_rem.setter
     def tc_rem(self,val):
         raise Exception('El atributo .tc_rem es de sólo lectura')
+
+    @property
+    def tc_min(self):
+        return self.__tc_min
+    
+    @tc_min.setter
+    def tc_min(self,val):
+        raise Exception('El atributo .tc_min es de sólo lectura')
     
     #
     # Funciones destinadas al Tipo de Cambio
@@ -87,22 +102,58 @@ class BCRA():
     def cargar_tc_ultimo_dia_habil(self,devolver=False):
         
         if self.tc_dia is None:
-            self.obtener_tc_bcra_3500(devolver=False)
+            self.cargar_tc_diario(devolver=False)
         
         if self.tc_udh is None:
             self.__tc_udh = (self.tc_dia
                 .groupby([self.tc_dia.index.year,self.tc_dia.index.month])
                 .max()
-                .pipe(lambda df_: df_.set_index(
+                .pipe(lambda adf: adf.set_index(
                     pd.DatetimeIndex(
-                        data = (dt.date(year=y,month=m,day=1) for y,m in df_.index),
+                        data = (dt.date(year=y,month=m,day=1) for y,m in adf.index),
                         name='Fecha'))
                 )
             )
             
         if devolver:
             return self.tc_udh
-    
+
+    def cargar_tc_minorista(self,devolver=False):
+        
+        try:
+            self.__tc_min = (pd
+                .read_html(
+                    bs(requests
+                        .get('https://www.bcra.gob.ar/publicacionesestadisticas/' + 
+                            'Planilla_cierre_de_cotizaciones.asp?dato=1&moneda=2')
+                        .text,
+                        features="lxml"
+                    )
+                    .find('table')
+                    .prettify()
+                )
+                [0]
+                .droplevel(0,axis=1) 
+                .assign(
+                    Fecha = lambda adf: pd.to_datetime(adf.Fecha,format='%d-%m-%Y'),
+                    Año = lambda adf: adf.Fecha.dt.year,
+                    Mes = lambda adf: adf.Fecha.dt.month
+                )
+                .rename(columns={'Comprador':'Comp','Vendedor':'Vend'})
+                .assign(**(
+                    {c: lambda adf, c=c: adf[c].div(1000) for c in ['Comp','Vend']} |
+                    {f'{c}_Var': lambda adf, c=c: adf[c].div(adf[c].shift(-1)).sub(1) for c in ['Comp','Vend']}
+                    )
+                )
+                .loc[:,['Fecha','Año','Mes','Comp','Vend','Comp_Var','Vend_Var',]]
+            )
+        except:
+                print('No se pudo cargar el Tipo de Cambio según la comunicación 3500 A del BRCRA.')
+                self.__tc_dia = (pd.DataFrame(
+                    columns = ['Año','Mes','Comp','Vend','Comp_Var','Vend_Var'],
+                    index = pd.DatetimeIndex(pd.to_datetime([]),name='Fecha'))
+                )
+
     #
     # Funciones destinadas al Relevamiento de Expectativas del Mercado (REM)
     #
@@ -192,12 +243,17 @@ class BCRA():
 
     def __explotar_proyeccion(self,fila_ini,fila_fin,n):
 
+        if n == 12:
+            n = 6; k = 12
+        else:
+            n = 12; k = 24
+        
         data = {
             'Fecha_REM': [fila_fin.Fecha_REM]*n,
             'Tabla': [fila_fin.Tabla]*n,
             'Unidad': [fila_fin.Unidad]*n,
-            'Periodo': fechas.lista_mensual(fila_ini.Periodo,n)[1:],
-            'Referencia': ['BLCTools: mensual']*n,
+            'Periodo': fechas.lista_mensual(fila_ini.Periodo, k)[(1 if n == 6 else 13):(7 if n == 6 else None)],
+            'Referencia': [f'Proy {"12" if n == 6 else "24"} meses']*n,
         }
 
         if 'IPC' in fila_ini.Tabla:
@@ -209,9 +265,9 @@ class BCRA():
             data |= {c: np.linspace(
                     start= fila_ini[c],
                     stop = fila_fin[c],
-                    num=n) 
+                    num=n+1)[1:] 
                 for c in fila_ini['Mediana':].index}
-            
+        
         return pd.DataFrame(data)
 
     def __explotar_proyecciones(self):
@@ -219,35 +275,22 @@ class BCRA():
         #Explotar proyecciones 12 meses
         dfs_12 = []
         for i, fila in self.rem.query('Periodo == "próx. 12 meses"').iterrows():
-            
             dfs_12.append(self.__explotar_proyeccion(
                 fila_ini=self.rem.loc[i-1,:],
                 fila_fin=fila,
-                n=6))
-
-        self.__rem = (pd
-            .concat([self.rem,] + dfs_12 )
-            .sort_values(by=['Fecha_REM','Tabla','Periodo'])
-            .query('Periodo != "próx. 12 meses"')
-            .reset_index(drop=True)
-        )
+                n=12))
         
         dfs_24 = []
         for i, fila in self.rem.query('Periodo == "próx. 24 meses"').iterrows():
-            
-            fila_ini = (self.rem
-                .query(f'Tabla == "{fila.Tabla}" and Referencia == "BLCTools: mensual"')
-                .iloc[-1,:])
-            
             dfs_24.append(self.__explotar_proyeccion(
-                fila_ini=fila_ini,
+                fila_ini=self.rem.loc[i-8,:],
                 fila_fin=fila,
-                n=12))
+                n=24))
         
         self.__rem = (pd
-            .concat([self.rem,] + dfs_24)
+            .concat([self.rem,] + dfs_12 + dfs_24)
             .sort_values(by=['Fecha_REM','Tabla','Periodo'])
-            .query('Periodo != "próx. 24 meses"')
+            .query('Periodo != "próx. 12 meses" & Periodo != "próx. 24 meses"')
             .reset_index(drop=True)
         )
 
