@@ -823,7 +823,7 @@ class TablasVC(SQLConnector):
         
         # Valida que si se colocaron ambas fechas, la más antigua sea la fecha de inicio
         if fecha_i and fecha_f:
-            fecha_i,fecha_f = fechas.validar_fechas(fecha_ini = fecha_i, fecha_fin = fecha_f)
+            fecha_i,fecha_f = fechas.validar_fechas(fecha_i, fecha_f)
             
         if query is None:
             query = f"SELECT * FROM incidencias WHERE evStFecha <= '{fecha_f}' AND (evEndFecha >= '{fecha_i}' OR evEndFecha IS NULL)"
